@@ -1,9 +1,12 @@
 """Coordinates and projection constants and helpers"""
 import logging
-from pyproj import Proj
+from pyproj import Proj, Geod
 from pyproj import transform
 
+from django.contrib.gis.geos import Point
+
 from lizard_map.models import BackgroundMap
+from lizard_map.models import Setting
 
 
 logger = logging.getLogger(__name__)
@@ -41,13 +44,67 @@ DEFAULT_MAP_SETTINGS = {
 rd_projection = Proj(RD)
 google_projection = Proj(GOOGLE)
 wgs84_projection = Proj(WGS84)
+geodesic = Geod('+ellps=sphere')
 
 srs_to_mapnik_projection = {
     'EPSG:28992': RD,
     'EPSG:900913': GOOGLE,
+    'EPSG:3857': GOOGLE,
     'EPSG:4326': WGS84,
     }
 
+string_to_srs = {
+    'rd': 'EPSG:28992',
+    'google': 'EPSG:900913',
+    'wgs84': 'EPSG:4326',
+}
+
+string_to_srid = {
+    'rd': 28992,
+    'google': 900913,
+    'wgs84': 4326,
+}
+
+srs_to_string = {
+    'EPSG:28992': "rd",
+    'EPSG:900913': "google",
+    'EPSG:3857': "google",
+    'EPSG:4326': "wgs84",
+}
+
+
+def transform_point(x, y, from_proj=None, to_proj=None):
+    """Transform x and y from from_proj to to_proj. Return a Point
+    with the right srid set.
+
+    Possible values of from_proj and to_proj are "google", "rd"
+    and "wgs84".
+
+    If from_proj or to_project aren't given, the "projection" Setting
+    is used.  It makes no sense to give neither."""
+
+    if to_proj is None:
+        to_srs = Setting.get('projection', 'EPSG:900913')
+        to_srid = string_to_srid[srs_to_string[to_srs]]
+        to_proj = Proj(srs_to_mapnik_projection[to_srs])
+    elif to_proj not in string_to_srs:
+        raise ValueError("Value '%s' of to_proj invalid." % to_proj)
+    else:
+        to_srid = string_to_srid[to_proj]
+        to_proj = Proj(srs_to_mapnik_projection[string_to_srs[to_proj]])
+
+    if from_proj is None:
+        from_proj = Setting.get('projection', 'EPSG:900913')
+        from_proj = Proj(srs_to_mapnik_projection[from_proj])
+    elif from_proj not in string_to_srs:
+        raise ValueError("Value '%s' of from_proj invalid." % from_proj)
+    else:
+        from_proj = Proj(srs_to_mapnik_projection[string_to_srs[from_proj]])
+
+    p = Point(*transform(from_proj, to_proj, x, y))
+    p.srid = to_srid
+    return p
+
 
 def google_to_rd(x, y):
     """Return RD coordinates from GOOGLE coordinates."""
@@ -70,12 +127,12 @@ def google_to_wgs84(x, y):
 
 
 def rd_to_wgs84(x, y):
-    """Return GOOGLE coordinates from RD coordinates."""
+    """Return WGS84 coordinates from RD coordinates."""
     return transform(rd_projection, wgs84_projection, x, y)
 
 
 def wgs84_to_rd(x, y):
-    """Return GOOGLE coordinates from RD coordinates."""
+    """Return WGS84 coordinates from RD coordinates."""
     return transform(wgs84_projection, rd_projection, x, y)
 
 
@@ -110,3 +167,6 @@ def detect_prj(prj):
     if 'GCS_WGS_1984' in prj:
         return WGS84
     return RD
+
+def translate_coords(lons, lats, az, dist):
+    return geodesic.fwd(lons, lats, az, dist)
