@@ -1,27 +1,34 @@
 import StringIO
+import csv
 import datetime
+import logging
+import urllib2
 
 from django.conf import settings
+from django.core.urlresolvers import reverse
 from django.db import transaction
 from django.db.models import Max
 from django.http import HttpResponse
 from django.http import HttpResponseBadRequest
-from django.http import HttpResponseForbidden
 from django.shortcuts import get_object_or_404
 from django.shortcuts import render
-from django.template.loader import render_to_string
+from django.shortcuts import redirect
 from django.template import RequestContext
+from django.template.loader import render_to_string
 from django.utils import simplejson as json
+from django.utils.translation import ugettext as _
 from django.views.decorators.cache import never_cache
-from django.views.generic.edit import FormView
 from django.views.generic.base import TemplateView
 from django.views.generic.base import View
+from django.views.generic.edit import FormView
+from djangorestframework.views import View as JsonView
+from lizard_ui.layout import Action
+from lizard_ui.models import ApplicationIcon
+from lizard_ui.views import UiView
+from lizard_ui.views import IconView
 import Image
-import csv
 import iso8601
-import logging
 import mapnik
-import urllib2
 
 #from lizard_map.daterange import deltatime_range
 #from lizard_map.daterange import store_timedelta_range
@@ -32,6 +39,7 @@ from lizard_map.adapter import parse_identifier_json
 from lizard_map.animation import AnimationSettings
 from lizard_map.animation import slider_layout_extra
 from lizard_map.coordinates import DEFAULT_OSM_LAYER_URL
+from lizard_map.coordinates import transform_point
 from lizard_map.dateperiods import ALL
 from lizard_map.dateperiods import MONTH
 from lizard_map.daterange import compute_and_store_start_end
@@ -50,16 +58,16 @@ from lizard_map.models import CollageEdit
 from lizard_map.models import CollageEditItem
 from lizard_map.models import Setting
 from lizard_map.models import WorkspaceEdit
+from lizard_map.models import WorkspaceEditItem
 from lizard_map.models import WorkspaceStorage
+from lizard_map.models import WorkspaceStorageItem
 from lizard_map.utility import analyze_http_user_agent
-from lizard_ui.models import ApplicationScreen
+from lizard_map.lizard_widgets import Legend
 from lizard_ui.views import ViewContextMixin
 
-
 CUSTOM_LEGENDS = 'custom_legends'
 MAP_LOCATION = 'map_location'
 MAP_BASE_LAYER = 'map_base_layer'  # The selected base layer
-CRUMBS_HOMEPAGE = {'name': 'home', 'title': 'hoofdpagina', 'url': '/'}
 TIME_BETWEEN_VIDEO_POPUP = datetime.timedelta(days=1)
 
 
@@ -84,7 +92,9 @@ class GoogleTrackingMixin(object):
 class WorkspaceMixin(object):
     """Add workspace and map variables.
     """
+    # Override with an empty string for no click handler.
     javascript_click_handler = 'popup_click_handler'
+    # javascript_click_handler = ''
 
     def workspace(self):
         """Implement a function that returns a workspace-storage,
@@ -134,6 +144,10 @@ class MapMixin(object):
         return s
 
     def start_extent(self):
+        # Hack: we need to have a session right away for toggling ws items.
+        self.request.session[
+            'make_sure_session_is_initialized'] = 'hurray'
+        # End of the hack.
         map_location = Setting.extent(
             'start_extent',
             DEFAULT_START_EXTENT)
@@ -155,8 +169,7 @@ class MapMixin(object):
     def base_layer_name(self):
         if MAP_BASE_LAYER in self.request.session:
             return self.request.session[MAP_BASE_LAYER]
-        else:
-            return ""
+        return ""
 
     @property
     def maps(self):
@@ -168,24 +181,19 @@ class MapMixin(object):
         # For the client side to determine is there is a google map.
         if self.maps.filter(
             layer_type=BackgroundMap.LAYER_TYPE_GOOGLE).count() > 0:
-
             return True
-        else:
-            return False
+        return False
 
     def background_maps(self):
         if self.maps:
             return self.maps
-        else:
-            logger.warn("No background maps are active. Taking default.")
-            maps = [BackgroundMap(
-                name='Default map',
-                default=True,
-                active=True,
-                layer_type=BackgroundMap.LAYER_TYPE_OSM,
-                layer_url=DEFAULT_OSM_LAYER_URL), ]
-
-            return maps
+        logger.warn("No background maps are active. Taking default.")
+        return [BackgroundMap(
+                    name='Default map',
+                    default=True,
+                    active=True,
+                    layer_type=BackgroundMap.LAYER_TYPE_OSM,
+                    layer_url=DEFAULT_OSM_LAYER_URL), ]
 
 
 class CollageMixin(object):
@@ -217,16 +225,139 @@ class DateRangeMixin(object):
         return DateRangeForm(self.current_date_range())
 
 
-class AppView(
-    WorkspaceEditMixin, CollageMixin, DateRangeMixin,
-    ViewContextMixin, MapMixin,
-    GoogleTrackingMixin, TemplateView):
-    """All-in-one"""
-    pass
+class CrumbsMixin(object):
+    def find_app_description(self, url):
+        """An App doesn't generally know what it is called on the
+        current site.  E.g., if the front page has an app called
+        "Metingen" that links to "/fews/fewsjdbc/almere/", then the
+        breadcrumb should show "home > metingen" instead of "home >
+        fewsjdbc > almere", but the app doesn't know this.
+
+        This function tries to help. It returns a tuple with three
+        elements:
+        - A list of breadcrumbs consisting of app screens (other than
+          Home), the last item of which leads to this app
+        - The part of the URL that led to this guess
+        - The rest of the URL (that a view can use to build further
+          parts of the breadcrumbs list)
+
+        If there are multiple screens that appear to fit, we use the
+        one that "uses up" most of the URL.
+        """
+
+        maxlength = None
+        found = (None, None, None)
+
+        if not url.startswith('/'):
+            url = '/' + url
+
+        for icon in ApplicationIcon.objects.all():
+            iconurl = icon.url
+            if iconurl.startswith('http://'):
+                continue
+            if not iconurl.startswith('/'):
+                iconurl = '/' + iconurl
+
+            if url.startswith(iconurl):
+                if maxlength is not None and len(iconurl) < maxlength:
+                    continue
+
+                crumb = {'url': icon.url,
+                         'description': icon.name,
+                         'title': icon.description or icon.name}
+
+                maxlength = len(iconurl)
+                found = (icon.application_screen.crumbs() + [crumb],
+                         iconurl,
+                         url[len(iconurl):])
+
+        return found
+
+    def crumbs(self):
+        """Returns a list of dictionaries, with keys
+        'url', 'description' and 'title'. Views in Apps should
+        override this function, use super() to get this default, and
+        then add their own crumbs."""
+        pass
+
+        initial = [{
+                'url': '/',
+                'description': 'Home',
+                'title': _('Back to homepage')}]
+
+        toapp, self.url_to_app, self.url_after_app = (
+            self.find_app_description(self.request.path))
+
+        if toapp:
+            return initial + toapp
+        else:
+            return initial
+
+
+class AppView(WorkspaceEditMixin, GoogleTrackingMixin, CollageMixin, DateRangeMixin, MapMixin,
+              UiView):
+    """Main map view (using twitter bootstrap). Replaces AppView."""
+
+    @property
+    def show_secondary_sidebar_title(self):
+        return _('Layers')
+
+    show_secondary_sidebar_icon = 'icon-list'
+
+    @property
+    def show_rightbar_title(self):
+        return _('Legend')
+
+    @property
+    def legends(self):
+        """Return legends for the rightbar."""
+        result = []
+        workspace_items = self.workspace().workspace_items.filter(
+            visible=True).reverse()
+        for workspace_item in workspace_items:
+            logger.debug("Looking for legend url for %s...", workspace_item)
+            if not hasattr(workspace_item.adapter, 'legend_image_url'):
+                logger.debug(
+                    "No legend_image_url() on this ws item's adapter.")
+                continue
+            result.append(Legend(
+                    name=workspace_item.name,
+                    image_url=workspace_item.adapter.legend_image_url()))
+        return result
+
+    @property
+    def content_actions(self):
+        """Add default-location-zoom."""
+        actions = super(MapView, self).content_actions
+        zoom_to_default = Action(
+            name=_('Default zoom'),
+            description=_('Zoom to default location'),
+            url=reverse('lizard_map.map_location_load_default'),
+            icon='icon-screenshot',
+            klass='map-load-default-location')
+        actions.insert(0, zoom_to_default)
+        set_date_range = Action(
+            name=_('Verander datumbereik'),
+            description=_('Verander het datumbereik van de metingen.'),
+            url=reverse('lizard_map_date_range'),
+            icon='icon-time',
+            klass='popup-date-range')
+        actions.insert(1, set_date_range)
+        activate_multiselect = Action(
+            name=_('Multi-select'),
+            description=_('Selecteer meerdere items.'),
+            url="javascript:void(null)",
+            icon='icon-multi-select',
+            klass='map-multiple-selection')
+        actions.insert(2, activate_multiselect)
+        return actions
+
+
+MapView = AppView  # BBB
 
 
 class WorkspaceStorageListView(
-    ViewContextMixin, GoogleTrackingMixin, TemplateView):
+    UiView, GoogleTrackingMixin):
     """Show list of storage workspaces."""
 
     template_name = 'lizard_map/workspace_storage_list.html'
@@ -235,73 +366,41 @@ class WorkspaceStorageListView(
         return WorkspaceStorage.objects.all()
 
 
-class WorkspaceStorageView(
-    WorkspaceMixin, CollageMixin, DateRangeMixin,
-    ViewContextMixin, MapMixin,
-    GoogleTrackingMixin, TemplateView):
-    """Workspace storage view.
-
-    TODO: "load workspace in my workspace and go there" """
+class WorkspaceStorageView(AppView):
+    """Workspace storage view."""
     template_name = 'lizard_map/workspace_storage_detail.html'
+    show_secondary_sidebar_title = False  # Don't show the 'layers' button.
 
+    @property
     def workspace(self):
         """Return a workspace"""
         if not hasattr(self, '_workspace'):
-            self._workspace = WorkspaceStorage.objects.get(
-                pk=self.workspace_id)
+            if self.workspace_id:
+                self._workspace = WorkspaceStorage.objects.get(
+                    pk=self.workspace_id)
+            elif self.workspace_slug:
+                self._workspace = WorkspaceStorage.objects.get(
+                    secret_slug=self.workspace_slug)
         return self._workspace
 
     def get(self, request, *args, **kwargs):
-        self.workspace_id = kwargs['workspace_id']
+        self.workspace_id = kwargs.get('workspace_id', None)
+        self.workspace_slug = kwargs.get('workspace_storage_slug', None)
         return super(WorkspaceStorageView, self).get(
             request, *args, **kwargs)
 
+    @property
+    def page_title(self):
+        return self.workspace.name
 
-class HomepageView(AppView):
-    """
-    Homepage view with apps on the left side
-
-    Try to fetch GET parameter 'screen' from url. It points to the
-    application_screen_slug.
-    """
-    template_name = 'lizard_map/app_screen.html'
-
-    def get_context_data(self, **kwargs):
-        context = super(HomepageView, self).get_context_data(**kwargs)
-
-        # Application screen slug
-        application_screen_slug = kwargs.get('application_screen_slug', None)
-        if application_screen_slug is None:
-            application_screen_slug = self.request.GET.get('screen', None)
-        context['application_screen_slug'] = application_screen_slug
-
-        # Breadcrumbs
-        crumbs_prepend = kwargs.get('crumbs_prepend', None)
-        if crumbs_prepend is None:
-            if application_screen_slug:
-                application_screen = get_object_or_404(
-                    ApplicationScreen,
-                    slug=application_screen_slug)
-                crumbs = [application_screen.crumb(), ]
-                if application_screen_slug != 'home':
-                    # prepend with "home"
-                    application_screen_home = get_object_or_404(
-                        ApplicationScreen,
-                        slug='home')
-                    crumbs = [application_screen_home.crumb(), ] + crumbs
-            else:
-                crumbs = [CRUMBS_HOMEPAGE]
-        else:
-            crumbs = list(kwargs['crumbs_prepend'])
-        context['crumbs'] = crumbs
-
-        return context
+    @property
+    def breadcrumbs(self):
+        """Return homepage + ourselves as breadcrumbs."""
+        result = [self.home_breadcrumb_element,
+                  Action(name=self.workspace.name)]
+        return result
 
 
-##### Edits on workspace ############
-
-
-# L3
 class ActionDialogView(ViewContextMixin, FormView):
     """
     Generic Action Dialog View.
@@ -379,15 +478,26 @@ class WorkspaceSaveView(ActionDialogView):
         workspace_edit = WorkspaceEdit.get_or_create(
            self.request.session.session_key, self.request.user)
         # TODO: quota, warning on duplicate names.
+
+        # For the initial release of Lizard 3, turn required authorization
+        # OFF.
         user = self.request.user
         if not user.is_authenticated():
-            html = render_to_string(
-                self.template_name_forbidden,
-                {'message': ('U kunt geen workspace opslaan als U '
-                             'niet bent ingelogd.')},
-                context_instance=RequestContext(self.request))
-            return HttpResponseForbidden(html)
-        workspace_edit.save_to_storage(name=form_data['name'], owner=user)
+            user = None
+            # html = render_to_string(
+            #     self.template_name_forbidden,
+            #     {'message': ('U kunt geen workspace opslaan als U '
+            #                  'niet bent ingelogd.')},
+            #     context_instance=RequestContext(self.request))
+            # return HttpResponseForbidden(html)
+        logger.debug("Before secret slug.")
+        secret_slug = (workspace_edit.
+                       save_to_storage(name=form_data['name'], owner=user))
+        logger.debug("After secret slug. slug=%s" % (secret_slug,))
+
+        self.saved_workspace_url = self.request.build_absolute_uri(
+            reverse('lizard_map_workspace_slug_storage',
+                    kwargs={'workspace_storage_slug': secret_slug}))
 
 
 class WorkspaceLoadView(ActionDialogView):
@@ -407,6 +517,10 @@ class WorkspaceLoadView(ActionDialogView):
         # TODO: check permissions.
         workspace_storage = WorkspaceStorage.objects.get(pk=form_data['id'])
         workspace_edit.load_from_storage(workspace_storage)
+        redirect = {
+            "redirect": reverse("lizard_ui.icons")
+        }
+        return HttpResponse(json.dumps(redirect))
 
 
 class DateRangeView(DateRangeMixin, WorkspaceEditMixin, ActionDialogView):
@@ -489,7 +603,7 @@ class CollageItemEditorView(ActionDialogView):
         # Select group to update the parameters.
         # Note: we also update invisible items
         grouped_collage_items, collage_item_group = group_collage_items(
-            CollageEditItem.objects.all())
+            CollageEditItem.objects.filter(collage=collage_item.collage))
 
         grouping_hint = collage_item_group[collage_item.id]
         collage_items = grouped_collage_items[grouping_hint]
@@ -557,7 +671,7 @@ def workspace_item_reorder(
         workspace_items_order = dict([
                 (workspace_item_id, index * 10) for
                 index, workspace_item_id in enumerate(
-                    request.POST.getlist('workspace-items[]'))])
+                    request.POST.getlist('workspace-item[]'))])
 
     for workspace_item in workspace_edit.workspace_items.all():
         workspace_item.index = workspace_items_order.get(
@@ -585,11 +699,9 @@ def workspace_item_toggle(
     if workspace_edit is None:
         workspace_edit = WorkspaceEdit.get_or_create(
             request.session.session_key, request.user)
-
     name = request.POST['name']
     adapter_class = request.POST['adapter_class']
     adapter_layer_json = request.POST['adapter_layer_json']
-
     # Find out if it is already present.
     existing_workspace_items = workspace_edit.workspace_items.filter(
         adapter_class=adapter_class,
@@ -665,24 +777,36 @@ def workspace_item_delete(request, workspace_edit=None, object_id=None):
     return HttpResponse(json.dumps(deleted))
 
 
-# #To be updated/tested
-# @never_cache
-# def workspace_item_extent(request, workspace_item_id=None):
-#     """Returns extent for the workspace in json.
+@never_cache
+def workspace_item_extent(request, item_class=WorkspaceEditItem):
+    """Return extent for the workspace item in json.
 
-#     Transform to correct client-side projection, then return coordinates.
-#     """
-#     workspace_item_id = request.GET['workspace_item_id']
-#     workspace_item = get_object_or_404(WorkspaceItem, pk=workspace_item_id)
-#     extent = workspace_item.adapter.extent()
+    Transform to correct client-side projection, then return coordinates.
 
-#     srs = Setting.get('projection', DEFAULT_PROJECTION)
-#     extent_converted['east'], extent_converted['north'] = google_to_srs(
-#         extent['east'], extent['north'], srs)
-#     extent_converted['west'], extent_converted['south'] = google_to_srs(
-#         extent['west'], extent['south'], srs)
+    With ``item_class`` we can customize it in
+    ``saved_workspace_item_extent()``.
 
-#     return HttpResponse(json.dumps(extent_converted))
+    """
+
+    workspace_item_id = request.GET['workspace_item_id']
+    workspace_item = get_object_or_404(item_class, pk=workspace_item_id)
+    extent = workspace_item.adapter.extent()
+    peastnorth = transform_point(extent['east'], extent['north'],
+                                 from_proj='google')
+    pwestsouth = transform_point(extent['west'], extent['south'],
+                                 from_proj='google')
+    return HttpResponse(json.dumps({
+                'east': peastnorth.get_x(),
+                'north': peastnorth.get_y(),
+                'west': pwestsouth.get_x(),
+                'south': pwestsouth.get_y(),
+                }))
+
+@never_cache
+def saved_workspace_item_extent(request):
+    """Return extent for the *saved* workspace item in json.
+    """
+    return workspace_item_extent(request, item_class=WorkspaceStorageItem)
 
 
 def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
@@ -692,7 +816,17 @@ def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
     everything.
 
     found: list of dictionaries {'distance': ..., 'timeserie': ...,
-    'workspace_item': ..., 'identifier': ...}.
+    'workspace_item': ..., 'identifier': ..., ['grouping_hint'...]}.
+
+    If 'grouping_hint' is given, that is used to group items,
+    otherwise the workspace_item.id. This way a single workspace item
+    can have things show up in different tabs. Please don't use
+    grouping_hints that could possibly come from other workspace items
+    (use the workspace item id in the hint).
+
+    The maximum number of tabs in popups can be configured via
+    POPUP_MAX_TABS in your site's settings.py. POPUP_MAX_TABS
+    is optional and defaults to 3.
 
     Note: identifier must be a dict. {'id': the_real_id}.
 
@@ -714,12 +848,16 @@ def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
     display_groups = {}
     display_group_order = []
     for display_object in found:
-        workspace_item = display_object['workspace_item']
-        if workspace_item.id not in display_groups:
-            display_groups[workspace_item.id] = []
-        display_groups[workspace_item.id].append(display_object)
-        if workspace_item.id not in display_group_order:
-            display_group_order.append(workspace_item.id)
+        if 'grouping_hint' in display_object:
+            key = display_object['grouping_hint']
+        else:
+            key = display_object['workspace_item'].id
+
+        if key not in display_groups:
+            display_groups[key] = []
+        display_groups[key].append(display_object)
+        if key not in display_group_order:
+            display_group_order.append(key)
 
     if len(display_groups) > 1:
         big_popup = True
@@ -727,7 +865,7 @@ def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
         big_popup = False
 
     # Now display them.
-    for workspace_item_id, display_group in display_groups.items():
+    for key, display_group in display_groups.items():
         # There MUST be at least one item in the group
         workspace_item = display_group[0]['workspace_item']
 
@@ -758,9 +896,10 @@ def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
 
         # if 'google_coords' in display_object:
         #     x_found, y_found = display_object['google_coords']
-        html[workspace_item.id] = html_per_workspace_item
+        html[key] = html_per_workspace_item
 
-    result_html = [html[index] for index in display_group_order][:3]
+    POPUP_MAX_TABS = getattr(settings, 'POPUP_MAX_TABS', 3)
+    result_html = [html[key] for key in display_group_order][:POPUP_MAX_TABS]
 
     if popup_id is None:
         popup_id = 'popup-id'
@@ -899,20 +1038,23 @@ Map stuff
 """
 
 
-def wms(request, workspace_storage_id=None):
+def wms(request, workspace_storage_id=None, workspace_storage_slug=None):
     """Return PNG as WMS service for given workspace_edit or
     workspace_storage.
 
-    if workspace_storage_id is not provided, it will take your own
-    WorkspaceEdit.
+    if workspace_storage_id and workspace_storage_slug are both not
+    provided, it will take your own WorkspaceEdit.
     """
 
-    if workspace_storage_id is None:
-        workspace = WorkspaceEdit.get_or_create(
-            request.session.session_key, request.user)
-    else:
+    if workspace_storage_id is not None:
         workspace = get_object_or_404(
             WorkspaceStorage, pk=workspace_storage_id)
+    elif workspace_storage_slug is not None:
+        workspace = get_object_or_404(
+            WorkspaceStorage, secret_slug=workspace_storage_slug)
+    else:
+        workspace = WorkspaceEdit.get_or_create(
+            request.session.session_key, request.user)
 
     # WMS standard parameters
     width = int(request.GET.get('WIDTH'))
@@ -994,7 +1136,10 @@ def search(workspace, google_x, google_y, radius):
 
 
 # L3
-def search_coordinates(request, workspace_storage_id=None, _format='popup'):
+def search_coordinates(request,
+                       workspace_storage_id=None,
+                       workspace_storage_slug=None,
+                       _format='popup'):
     """searches for objects near GET x,y,radius returns json_popup of
     results.
 
@@ -1013,20 +1158,27 @@ def search_coordinates(request, workspace_storage_id=None, _format='popup'):
     x = float(request.GET.get('x'))
     y = float(request.GET.get('y'))
     format = request.GET.get('format', _format)
+
     # TODO: convert radius to correct scale (works now for google + rd)
     radius = float(request.GET.get('radius'))
     radius_search = radius
+
     if 'HTTP_USER_AGENT' in request.META:
         analyzed_user_agent = analyze_http_user_agent(
             request.META['HTTP_USER_AGENT'])
         # It's more difficult to point with your finger than with the mouse.
         if analyzed_user_agent['device'] == 'iPad':
             radius_search = radius_search * 3
+
     srs = request.GET.get('srs')
     google_x, google_y = coordinates.srs_to_google(srs, x, y)
 
-    if workspace_storage_id:
+    # Load correct workspace, several possibilities
+    if workspace_storage_id is not None:
         workspace = WorkspaceStorage.objects.get(pk=workspace_storage_id)
+    elif workspace_storage_slug is not None:
+        workspace = WorkspaceStorage.objects.get(
+            secret_slug=workspace_storage_slug)
     else:
         user_workspace_id = request.GET.get('user_workspace_id', None)
         if user_workspace_id is not None:
@@ -1035,7 +1187,9 @@ def search_coordinates(request, workspace_storage_id=None, _format='popup'):
             stored_workspace_id = request.GET.get('stored_workspace_id', None)
             workspace = WorkspaceStorage.objects.get(pk=stored_workspace_id)
 
+    # The actual search!
     found = search(workspace, google_x, google_y, radius)
+    logger.debug('>>> FOUND <<< %s\n%s' % (format, repr(found)))
 
     if found:
         # ``found`` is a list of dicts {'distance': ..., 'timeserie': ...}.
@@ -1070,7 +1224,7 @@ def search_coordinates(request, workspace_storage_id=None, _format='popup'):
 
 
 class CollageDetailView(
-    CollageMixin, DateRangeMixin, ViewContextMixin, TemplateView):
+    CollageMixin, DateRangeMixin, UiView):
     """
     Shows "my collage" as big page.
     """
@@ -1091,8 +1245,7 @@ class CollageDetailView(
         return super(CollageDetailView, self).get(request, *args, **kwargs)
 
 
-class CollageStatisticsView(
-    ViewContextMixin, TemplateView):
+class CollageStatisticsView(UiView):
     """
     Shows statistics for collage items.
     """
@@ -1130,7 +1283,11 @@ class CollageView(CollageMixin, ActionDialogView):
         google_x, google_y = coordinates.srs_to_google(srs, x, y)
 
         # Workspace.
-        workspace = WorkspaceEdit.objects.get(pk=workspace_id)
+        if form_data['workspace_type'] == 'workspace_storage':
+            workspace = WorkspaceStorage.objects.get(pk=workspace_id)
+        else:
+            workspace = WorkspaceEdit.objects.get(pk=workspace_id)
+
         collage = CollageEdit.get_or_create(
             self.request.session.session_key, self.request.user)
 
@@ -1140,12 +1297,21 @@ class CollageView(CollageMixin, ActionDialogView):
             # Add all found items to collage.
             logger.debug("Adding collage item %s" % found_item['name'])
             #print '%r' % found_item['identifier']
-            collage.collage_items.create(
-                adapter_class=found_item['workspace_item'].adapter_class,
-                adapter_layer_json=found_item[
-                    'workspace_item'].adapter_layer_json,
-                name=found_item['name'][:80],
-                identifier=found_item['identifier'])
+
+            # Don't add them if they already exist
+            adapter_class = found_item['workspace_item'].adapter_class
+            adapter_layer_json = (found_item['workspace_item']
+                                  .adapter_layer_json)
+            name = found_item['name'][:80]
+            identifier = found_item['identifier']
+
+            if not collage.data_in_collage(adapter_class, adapter_layer_json,
+                                           name, identifier):
+                collage.collage_items.create(
+                    adapter_class=adapter_class,
+                    adapter_layer_json=adapter_layer_json,
+                    name=name,
+                    identifier=identifier)
 
 
 class CollageAddView(CollageMixin, ActionDialogView):
@@ -1160,13 +1326,25 @@ class CollageAddView(CollageMixin, ActionDialogView):
         data = form.cleaned_data
         collage = CollageEdit.get_or_create(
             self.request.session.session_key, self.request.user)
+
         # Parse_identifier_json is unnecessary, but it provides an
         # extra check.
+        adapter_class = data['adapter_class']
+        adapter_layer_json = data['adapter_layer_json']
+        name = data['name'][:80]
+        identifier = parse_identifier_json(data['identifier'])
+
+        if collage.data_in_collage(adapter_class,
+                                   adapter_layer_json,
+                                   name,
+                                   identifier):
+            return
+
         collage.collage_items.create(
-            adapter_class=data['adapter_class'],
-            adapter_layer_json=data['adapter_layer_json'],
-            name=data['name'][:80],
-            identifier=parse_identifier_json(data['identifier']))
+            adapter_class=adapter_class,
+            adapter_layer_json=adapter_layer_json,
+            name=name,
+            identifier=identifier)
 
 
 class CollageEmptyView(CollageView):
@@ -1617,7 +1795,38 @@ class AdapterImageView(AdapterMixin, ImageMixin, View):
             layout_extra=layout_extra)
 
 
-class AdapterValuesView(AdapterMixin, ViewContextMixin, TemplateView):
+class AdapterFlotGraphDataView(AdapterMixin, JsonView):
+    """
+    Return result of adapter.flot_graph_data, using given parameters.
+
+    URL GET parameters:
+    - adapter_class (required)
+    - identifier (required, multiple supported)
+    - start_date, end_date (optional, iso8601 format, default current)
+    - layout_extra (optional)
+    """
+
+    _IGNORE_IE_ACCEPT_HEADER = False # Keep this, if you want IE to work
+
+    def get(self, request, *args, **kwargs):
+        """
+        Note: named url arguments become kwargs.
+        """
+        current_adapter = self.adapter(kwargs['adapter_class'])
+        identifier_list = self.identifiers()
+
+        start_date, end_date = self.start_end_dates_from_request()
+
+        # Add animation slider position, info from session data.
+        layout_extra = self.layout_extra_from_request()
+        layout_extra.update(slider_layout_extra(self.request))
+
+        return current_adapter.flot_graph_data(
+            identifier_list, start_date, end_date,
+            layout_extra=layout_extra)
+
+
+class AdapterValuesView(AdapterMixin, UiView):
     """
     Return values for a single identifier in csv or html.
 
@@ -1641,7 +1850,7 @@ class AdapterValuesView(AdapterMixin, ViewContextMixin, TemplateView):
         self.name = adapter.location(**identifier).get('name', 'export')
 
         if output_type == 'csv':
-            filename = '%s.csv' % (self.name)
+            filename = ('%s.csv' % (self.name)).encode('us-ascii', errors='ignore')
             # Make the csv output.
             response = HttpResponse(mimetype='text/csv')
             response['Content-Disposition'] = (
@@ -1656,3 +1865,10 @@ class AdapterValuesView(AdapterMixin, ViewContextMixin, TemplateView):
             # Make html table using self.values
             return super(AdapterValuesView, self).get(
                 request, *args, **kwargs)
+
+
+class HomepageView(MapView, IconView):
+    template_name = 'lizard_map/icons.html'
+
+
+MapIconView = HomepageView  # BBB
